
const emptyTableHtml: string = `
    <div class="max-w-md w-full min-h-[400px] flex flex-col justify-center mx-auto px-6 py-4">
        <div class="flex justify-center items-center size-[46px] bg-gray-100 rounded-lg dark:bg-neutral-800">
            <svg class="shrink-0 size-6 text-gray-600 dark:text-neutral-400" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24" stroke="currentColor" stroke-width="2" fill="none" stroke-linecap="round" stroke-linejoin="round"><path d="M4 11a9 9 0 0 1 9 9"></path><path d="M4 4a16 16 0 0 1 16 16"></path><circle cx="5" cy="19" r="1"></circle></svg>
        </div>
        <h2 class="mt-5 font-semibold text-gray-800 dark:text-white">
            No results found
        </h2>
        <p class="mt-2 text-sm text-gray-600 dark:text-neutral-400">
            Create a filter and it will appear here.
        </p>

        <div class="mt-5 flex flex-col sm:flex-row gap-2">
            <button type="button" class="openFilterModal-js py-2 px-3 inline-flex justify-center items-center gap-x-2 text-sm font-medium rounded-lg border border-transparent bg-blue-600 text-white hover:bg-blue-700 focus:outline-hidden focus:bg-blue-700 disabled:opacity-50 disabled:pointer-events-none" data-hs-overlay="#filterModal">
                <svg class="shrink-0 size-4" xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"><path d="M5 12h14"/><path d="M12 5v14"/></svg>
                Create a filter
            </button>
            <button type="button" onclick="alert('not implemented');" class="py-2 px-3 inline-flex items-center gap-x-2 text-sm font-medium rounded-lg border border-gray-200 bg-white text-gray-800 shadow-xs hover:bg-gray-50 focus:outline-hidden focus:bg-gray-50 disabled:opacity-50 disabled:pointer-events-none dark:bg-neutral-800 dark:border-neutral-700 dark:text-white dark:hover:bg-neutral-700 dark:focus:bg-neutral-700">
                Use a Template
            </button>
        </div>
    </div>
`;


var table: any;
const defineTable = () => {
    table = new HSDataTable($("#table")[0], {
        ajax: {
            url: `/guild/${guildId}/filters/api/datatable`,
            dataSrc: "data",
            data: (d: any) => {
                if (d === undefined) { return ;}

                d.filters = {};
                const is_whitelist = $("input[name='filterType']:checked").val();
                d.filters.is_whitelist = is_whitelist;
            }
        },
        serverSide: true,
        processing: true,
        selecting: true,
        pagingOptions: {
            pageBtnClasses: "hidden"
        },
        rowSelectingOptions: {
            selectAllSelector: "#selectAllBox"
        },
        language: {
            zeroRecords: emptyTableHtml,
            emptyTable: emptyTableHtml,
            loading: "Placeholder Loading Message...",
        },
        rowCallback: (row: any, data: any, index: number) => {
            $(row).addClass("bg-white dark:bg-neutral-900");
        },
        drawCallback: () => {
            HSDropdown.autoInit();
        },
        select: {
            style: "multi",
            selector: "td:first-child input[type='checkbox']"
        },
        columnDefs: [
            {
                // Row select checkbox
                targets: 0,
                orderable: false,
                searchable: false,
                render: (data, type, row) => {
                    return `
                        <td class="size-px whitespace-nowrap">
                            <div class="ps-6 py-4">
                                <label for="rowSelect${row.id}-js" class="flex">
                                    <input type="checkbox" id="rowSelect${row.id}-js" class="form-checkbox shrink-0 border-gray-300 rounded-sm text-blue-600 focus:ring-blue-500 disabled:opacity-50 disabled:pointer-events-none dark:bg-neutral-800 dark:border-neutral-600 dark:checked:bg-blue-500 dark:checked:border-blue-500 dark:focus:ring-offset-gray-800" data-hs-datatable-row-selecting-individual="">
                                    <span class="sr-only">Checkbox</span>
                                </label>
                            </div>
                        </td>
                    `;
                }
            },
            {
                // Name
                targets: 1,
                data: "name",
                orderable: true,
                searchable: true,
                render: (data, _type, row) => {
                    return `
                        <td class="size-px whitespace-nowrap align-top">
                            <span class="openFilterModal-js block px-6 py-4 text-blue-500 hover:text-blue-600 focus:text-blue-600 dark:text-blue-400 dark:hover:text-blue-500 dark:focus:text-blue-500 text-nowrap cursor-pointer" data-id="${row.id}">
                                ${data}
                            </sp>
                        </td>
                    `;
                }
            },
            {
                // Match
                targets: 2,
                data: "match",
                orderable: true,
                searchable: true,
                render: data => {
                    return `
                        <td class="size-px whitespace-nowrap align-top">
                            <a href="#" class="block px-6 py-4 text-blue-500 hover:text-blue-600 focus:text-blue-600 dark:text-blue-400 dark:hover:text-blue-500 dark:focus:text-blue-500 text-nowrap">
                                ${data}
                            </a>
                        </td>
                    `;
                }
            },
            {
                // Algorithm
                target: 3,
                data: "algorithm",
                orderable: true,
                searchable: true,
                render: (data, type, row) => {
                    return `
                        <td class="size-px whitespace-nowrap align-top">
                            <a href="#" class="block px-6 py-4 text-blue-500 hover:text-blue-600 focus:text-blue-600 dark:text-blue-400 dark:hover:text-blue-500 dark:focus:text-blue-500 text-nowrap">
                                ${data}
                            </a>
                        </td>
                    `;
                }
            },
            {
                // Filters
                target: 4,
                data: "is_insensitive",
                orderable: true,
                searchable: true,
                render: data => {
                    wrapper = $("<div>").addClass("px-6 py-4");
                    badge = $("<span>").addClass("py-1 px-1.5 inline-flex items-center text-xs font-medium rounded-full");
                    label = $("<span>");

                    if (data) {
                        badge.addClass("bg-red-100 text-red-800 dark:bg-red-500/10 dark:bg-red-500");
                        badge.append(label.text("No"));
                    } else {
                        badge.addClass("bg-teal-100 text-teal-800 dark:bg-teal-500/10 dark:text-teal-500");
                        badge.append(label.text("Yes"));
                    }

                    wrapper.append(badge);
                    return wrapper.get(0);
                }
            },
            {
                // Whitelist
                target: 5,
                data: "is_whitelist",
                orderable: true,
                searchable: true,
                render: data => {
                    wrapper = $("<div>").addClass("px-6 py-4");
                    badge = $("<span>").addClass("py-1 px-1.5 inline-flex items-center text-xs font-medium rounded-full");
                    label = $("<span>");

                    if (data) {
                        badge.addClass("bg-teal-100 text-teal-800 dark:bg-teal-500/10 dark:text-teal-500");
                        badge.append(label.text("Whitelist"));
                    } else {
                        badge.addClass("bg-red-100 text-red-800 dark:bg-red-500/10 dark:bg-red-500");
                        badge.append(label.text("Blacklist"));
                    }

                    wrapper.append(badge);
                    return wrapper.get(0);
                }
            },
            {
                // Created At
                target: 6,
                data: "created_at",
                orderable: true,
                searchable: true,
                render: data => {
                    return `
                        <td class="size-px whitespace-nowrap align-top">
                            <div class="px-6 py-4">
                                <span class="text-sm text-gray-500 dark:text-neutral-500 text-nowrap">
                                    ${formatTimestamp(data)}
                                </span>
                            </div>
                        </td>
                    `;
                }
            }
        ]
    })

    table.dataTable
        .on("select", onTableSelectChange)
        .on("deselect", onTableSelectChange)
        .on("draw", onTableSelectChange);
}

// Ensure the datatable recognises when all rows are selected, otherwise rows are only visually selected
$("#selectAllBox").on("change", function() {
    this.checked ? table.dataTable.rows().select() : table.dataTable.rows().deselect();
});

const onTableSelectChange = () => {
    const selectedRowCount = table.dataTable.rows({ selected: true }).count();
    $("#deleteRowsBtn").prop("disabled", selectedRowCount === 0);
    $(".rows-selected-count-js").text(selectedRowCount);

    const $elem = $(".rows-selected-count-js.zero-empty-js");
    selectedRowCount === 0 ? $elem.hide() : $elem.show();
}

$(window).ready(() => {
    setTimeout(defineTable, 500);
});

$("input[name='filterType']").on("change", () => {
    table.dataTable.draw();
});

const openFilterForm = async id => {
    $("#filterForm").removeClass("submitted");

    const formAlgorithmSelect = HSSelect.getInstance("#formAlgorithm");
    formAlgorithmSelect.setValue("");

    if (id === -1) {
        $("#formName").val("");
        $("#formMatch").val("");
        $("#formWhitelist").prop("checked", false);
        $("#formInsensitive").prop("checked", false);
    } else {
        const data = await $.ajax({
            url: `/guild/${guildId}/filters/api?id=${id}`,
            method: "get"
        });

        $("#formName").val(data.name);
        $("#formMatch").val(data.match);
        $("#formWhitelist").prop("checked", data.is_whitelist);
        $("#formInsensitive").prop("checked", data.is_insensitive);
        formAlgorithmSelect.setValue(data.algorithm);
    }

    HSOverlay.open($("#filterModal").get(0));
}

const closeFilterForm = () => {
    $("#filterForm").removeClass("submitted");
    HSOverlay.close($("#filterModal").get(0));
}

$(document).on("click", ".openFilterModal-js", event => {
    openFilterForm($(event.target).data("id") || -1);
});

const submitForm = async event => {
    event.preventDefault();

    const form = $(event.target).get(0);
    $(form).addClass("submitted");

    if (!form.checkValidity()) { return; }

    await $.ajax({
        url: `/guild/${guildId}/filters/api`,
        method: "post",
        dataType: "json",
        data: $(event.target).serializeArray(),
        success: () => {
            table.dataTable.draw();
            closeFilterForm();
        },
        error: error => {
            alert(JSON.stringify(error, null, 4));
        }
    });
}

$("#filterForm").on("submit", submitForm);
