// Preline: necessary for header events.
window.addEventListener("load", () => {
    const inputs = document.querySelectorAll('.dt-container thead input');

    inputs.forEach(input => {
        (input as HTMLInputElement).addEventListener("keydown", (event: KeyboardEvent) => {
            if ((event.metaKey || event.ctrlKey) && event.key === "a") {
                (event.target as HTMLInputElement).select();
            }
        });
    });
});

/**
 * Formats a given timestamp to one of two formats depending on its age.
 * @param timestamp 
 * @returns 'DD MMM, HH:mm' if younger than 1 year, else 'DD MMM YYYY'
 */
const formatTimestamp = (timestamp: string | number) => {
    const date = new Date(
        typeof timestamp === "string"
        ? timestamp.replace(" ", "T")
        : timestamp
    );
    const now = new Date();
    const difference = now.getTime() - date.getTime();

    // Day and short month (example: 21 Oct)
    const result = `${date.getDate()} ${date.toLocaleString("en-GB", { month: "short" })}`

    // Difference is less than a year:      'DD MMM, HH:mm'
    // Or, difference is more than a year:  'DD MMM YYYY'
    return difference < 31536000000
        ? result + `, ${date.getHours().toString().padStart(2, "0")}:${date.getMinutes().toString().padStart(2, "0")}`
        : result + ` ${date.getFullYear()}`;
}