import path from "path";
import dotenv from "dotenv";

dotenv.config();  // I env vars are already loaded, but this file is invoked directly sometimes.

const {
    DB_CLIENT = "sqlite",                                   // Default:
    SQLITE_FILE = path.resolve(__dirname, "../db.sqlite"),  // use sqlite if not specified
    PG_HOST = "",
    PG_PORT = "",
    PG_USER = "",
    PG_PASSWORD = "",
    PG_DATABASE = ""
} = process.env;

const dbConfig = {
    sqlite: {
        client: "sqlite3",
        connection: {
            filename: SQLITE_FILE
        },
        useNullAsDefault: true
    },
    postgresql: {
        client: "pg",
        connection: {
            host: PG_HOST,
            port: PG_PORT,
            user: PG_USER,
            password: PG_PASSWORD,
            database: PG_DATABASE
        }
    }
}

const knexConfig = {
    ...dbConfig[DB_CLIENT as keyof typeof dbConfig],
    migrations: {
        tableName: "knex_migrations",
        directory: path.resolve(__dirname, "./db/migrations")
    },
    seeds: {
        directory: path.resolve(__dirname, "./db/seeds")
    }
}

export default knexConfig;