import { Request, Response, NextFunction } from "express";
import passport, { PassportStatic } from "passport"
import { Strategy, Profile } from "passport-discord";

export const get = async (_request: Request, response: Response) => {
    response.render("login", {
        title: "Login - Relay"
    });
};

export const authenticate = async (request: Request, response: Response, next: NextFunction) => {
    passport.authenticate("discord", {
        successRedirect: "/",
        failureRedirect: "/auth/login",
        failureFlash: true
    })(request, response, next);
};

export const logout = async (request: Request, response: Response, next: NextFunction) => {
    request.logout(error => {
        if (error) { return next(); }
        response.redirect("/");
    });
};

export const setupPassport = (passport: PassportStatic) => {
    const scopes: Array<string> = ["identify"];
    const authorisedUserIds: Array<string> = process.env.DISCORD_USER_IDS?.split("/") || [];
    passport.use(new Strategy({
        clientID: process.env.CLIENT_ID || "",
        clientSecret: process.env.CLIENT_SECRET || "",
        callbackURL: process.env.CALLBACK_URL || "",
        scope: scopes
    },
    (_accessToken: string, _refreshToken: string, profile: Profile, callback: CallableFunction) => {
        if (authorisedUserIds.includes(profile.id)) {
            return callback(null, profile);
        }

        return callback(null, false, {
            "message": "User ID is not authorised!"
        })
    }));

    passport.serializeUser((user, done) => {
        done(null, user);
    });

    passport.deserializeUser((user: any, done) => {
        done(null, user);
    });
}

export default { get, authenticate, logout };